@@ -1,10 +1,12 @@
-from tnetwork.dyn_graph import DynGraphSN,DynGraphIG
+#from tnetwork.dyn_graph import DynGraphSN,DynGraphIG
+from tnetwork.dyn_graph import *
 from tnetwork.dyn_community.communities_dyn_sn import DynCommunitiesSN
 from tnetwork.dyn_community.communities_dyn_ig import DynCommunitiesIG
 from tnetwork.readwrite import *
 from tnetwork.DCD import *
 from tnetwork.DCD.analytics.dynamic_partition import *
 from tnetwork.utils import Intervals
+
 #from tnetwork.experiments.experiments import generate_toy_random_network
 
-from tnetwork.visualization.plots import plot_as_graph, plot_longitudinal, plot_longitudinal_sn_clusters
\ No newline at end of file
+from tnetwork.visualization.plots import plot_as_graph, plot_longitudinal
\ No newline at end of file
